/** 
 *  @file    ControlAutoLayout.h
 *  @author  Yongsheng Huang
 *  @date    8/9/2016  
 *  @version 0.0.1
 *  
 *  @brief Layout control automatically in MFC dialog
 *
 *  @section DESCRIPTION  
 *  
 *  @how to
 *
 *  1Copy ControlAutoLayout.h, ControlAutoLayout.cpp, Layout.h too you project
 *  2Include ControlAutoLayout.h in the dialog class file
 *  3Add a member variable of ControlAutoLayout in dialog class,
 *      such as: ControlAutoLayout control_auto_layout_;
 *  4On the OnInitDialog() function of dialog class 
 *      use "control_auto_layout_.SetDialogHandle(this);"  to 
 *      set the dialog handle to ControlAutoLayout
 *  5On the OnSize() function of dialog class
 *      use "control_auto_layout_.InitLayout(int window_width, int window_height);" 
 *      to init ControlAutoLayout and then, set each control's change mode:
 *      control_auto_layout_.SetControlAutoChangeMode(int ctrl_id, int left_change_mode, 
 *      int right_change_mode, int top_change_mode, int bottom_change_mode);
 */

#pragma once
#include <cmath>
#include <map>
#include "Layout.h"

class ControlAutoLayout
{
public:
    ControlAutoLayout(void);
    ~ControlAutoLayout(void);


public:
    /** 
    *   @brief  Init the rect and calc the ratio   
    *  
    *   @param  window_width    the width of dialog
    *   @param  window_height   the height of dialog
    *   @return void
    */
    void InitLayout(int window_width, int window_height);

    /** 
    *   @brief  Set the control change mode   
    *  
    *   @param  ctrl_id             the id of control
    *   @param  left_change_mode    the mode how to change the left coord
    *   @param  right_change_mode   the mode how to change the right coord
    *   @param  top_change_mode     the mode how to change the top coord
    *   @param  bottom_change_mode  the mode how to change the bottom coord
    *   @return void
    */
    void SetControlAutoChangeMode(int ctrl_id, int left_change_mode, 
        int right_change_mode, int top_change_mode, int bottom_change_mode);

    /** 
    *   @brief  Set dialog handle to this class to call some function    
    *  
    *   @param  wnd_dialog    the handle of dialog which contains the control
    *   @return void
    */
    void SetDialogHandle(CWnd *wnd_dialog);

    /** 
    *   @brief  Enable the layout function or not    
    *  
    *   @param  is_auto_layout    indicate Enable layout or not
    *   @return void
    */
    void SetAutoLayoutEnable(bool is_auto_layout);

    /** 
    *   @brief  Update the original rect of control.
    *           It will be use when you change the control original size 
    *  
    *   @param  control_item    the handle of control
    *   @param  rect            new rect of control
    *   @return void
    */
    void UpateControlOriginalRect(CWnd* control_item, CRect rect);


private:
    void SetClientWindowRectOriginal(int window_width, int window_height);

    void SetClientWindowRectNow(int window_width, int window_height);

    void CalcHorizontalRatio();

    void CalcVerticalRatio();

    //
    void CalcHorizontalCoord(int left_change_mode, int right_change_mode,
        int &left_coord, int &right_coord);

    void CalcVerticalCoord(int top_change_mode, int bottom_change_mode,
        int &top_coord, int &bottom_coord);


    //////////////////////////////////////////////////////////////////////////
    void CalcLeftPadRightPad(int &left_coord, int &right_coord);

    void CalcLeftPadRightRatio(int &left_coord, int &right_coord);

    void CalcLeftRatioRightPad(int &left_coord, int &right_coord);

    void CalcLeftRatioRightRatio(int &left_coord, int &right_coord);

    void CalcLeftRatioRightFixedWidth(int &left_coord, int &right_coord);

    void CalcLeftFixedWidthRightPad(int &left_coord, int &right_coord);

    void CalcLeftFixedWidthRightRatio(int &left_coord, int &right_coord);

    //////////////////////////////////////////////////////////////////////////
    void CalcTopPadBottomPad(int &top_coord, int &bottom_coord);

    void CalcTopPadBottomRatio(int &top_coord, int &bottom_coord);

    void CalcTopRatioBottomPad(int &top_coord, int &bottom_coord);

    void CalcTopRatioBottomRatio(int &top_coord, int &bottom_coord);

    void CalcTopRatioBottomFixedHeight(int &top_coord, int &bottom_coord);

    void CalcTopFixedHeightBottomPad(int &top_coord, int &bottom_coord);

    void CalcTopFixedHeightBottomRatio(int &top_coord, int &bottom_coord);

    //////////////////////////////////////////////////////////////////////////
    void InitControlRectInMap(CWnd* control_item);

    CRect GetControlRect(CWnd* control_item);

    int GetHorizontalChangeMode(int left_change_mode, int right_change_mode);

    int GetVerticalChangeMode(int top_change_mode, int bottom_change_mode);

    void GetRectCoord(CRect rect, int &left_coord, int &right_coord,
        int &top_coord, int &bottom_coord);

    void SetRectByCoord(CRect &rect, int left_coord, int right_coord,
        int top_coord, int bottom_coord);

    bool IsExistControlRect(CWnd* control_item);

    bool IsEqual(double x, double y);

private:
    CWnd *wnd_dialog_;

    bool is_auto_layout_;

    CRect rect_client_window_original_;
    CRect rect_client_window_now_;

    double horizontal_ratio_;
    double vertical_ratio_;
    std::map<CWnd*,CRect> map_control_original_rect_;
};

